# -*- coding: utf-8 -*-
import os
from langchain.chat_models import ChatOpenAI
from langchain.agents import Tool, load_tools
from langchain.memory import ConversationBufferMemory
from langchain.agents import initialize_agent
from langchain.tools.python.tool import PythonREPLTool
from langchain.utilities import GoogleSearchAPIWrapper


class CopilotLLM:

    def __init__(self, verbose=True, additional_tools=[]):
        self.chain = get_chat_chain(verbose, additional_tools)

    def run(self, instructions):
        try:
            result = self.chain.run(instructions)
            return result
        except Exception as e:
            # Workaround for issue https://github.com/hwchase17/langchain/issues/1358.
            if "Could not parse LLM output:" in str(e):
                return str(e).split("Could not parse LLM output:")[1]
            else:
                raise e


def get_chat_chain(verbose=True, additional_tools=[], agent="chat-zero-shot-react-description", max_iterations=30):
    '''Initialize the LLM chain with useful tools.'''
    llm = ChatOpenAI(engine="gpt-35-turbo", max_tokens=512)

    tools = load_tools(["terminal", "human"], llm)
    if os.getenv("KUBE_COPILOT_ENABLE_PYTHON"):
        tools += [
            Tool(
                name="Python",
                func=PythonREPLTool().run,
                description="helps to run Python codes"
            )
        ]

    if os.getenv("GOOGLE_API_KEY") != "" and os.getenv("GOOGLE_CSE_ID") != "":
        tools += [
            Tool(
                name="Search",
                func=GoogleSearchAPIWrapper(
                    google_api_key=os.getenv("GOOGLE_API_KEY"),
                    google_cse_id=os.getenv("GOOGLE_CSE_ID"),
                ).run,
                description="search the web for current events or current state of the world"
            )
        ]

    tools += additional_tools

    memory = ConversationBufferMemory(
        memory_key="chat_history", return_messages=True)
    chain = initialize_agent(
        tools, llm, agent=agent, memory=memory,
        verbose=verbose, max_iterations=max_iterations)
    return chain



# -*- coding: utf-8 -*-

_base_prompt = '''Follow the below instructions to complete the tasks. Please ensure the tasks are within Kubernetes and cloud native networking domain. If any write or delete operations are quired, or you are not sure on the instructions, please invoke human tool to ask more inputs. Please do not try to install anything. If some tools are missing, just skip the instruction steps.'''


_base_diagnose_prompt = '''As a technical expert in Kubernetes and cloud native networking, your task is to diagnose and resolve questions and issues related to these technologies. You should have a deep understanding of the underlying principles of Kubernetes and cloud native networking, as well as experience troubleshooting common problems that may arise.

Your response should be detailed and provide step-by-step instructions on how to diagnose and resolve the issue at hand. You should also be able to communicate effectively with non-technical users, providing clear explanations of complex concepts and solutions. Please do not try to install anything. If some tools are missing, just skip the instruction steps.

Please note that you should be flexible enough to handle various scenarios involving Kubernetes and cloud native networking, including those related to deployment, scaling, security, monitoring, debugging, and optimization. Your goal is to provide accurate and effective solutions that help users overcome their technical challenges.

Here is the Pod {pod} in namespace {namespace} to diagnose.'''

_base_audit_prompt = '''As a technical expert in Kubernetes and cloud native security, your task is to audit the security issues related to these technologies. You should have a deep understanding of the underlying security principles of Kubernetes and cloud native, as well as experience troubleshooting common problems that may arise.

Your response should be detailed and provide step-by-step instructions on how to diagnose and resolve the issue at hand. You should also be able to communicate effectively with non-technical users, providing clear explanations of complex concepts and solutions. Please do not try to install anything. If some tools are missing, just skip the instruction steps.

Your goal is to provide accurate and effective solutions that help users overcome their security challenges, including CIS compliance, CVE, NSA & CISA Kubernetes Hardening Guidance and so on.

Here is the Pod {pod} in namespace {namespace} to audit.'''


def get_prompt(instruct):
    return f"{_base_prompt}\nHere are the instructions: {instruct}"

def get_diagnose_prompt(namespace, pod):
    return _base_diagnose_prompt.format(pod=pod, namespace=namespace)

def get_audit_prompt(namespace, pod):
    return _base_audit_prompt.format(pod=pod, namespace=namespace)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import click
from kube_copilot.llm import init_openai
from kube_copilot.agent import CopilotLLM
from kube_copilot.prompts import (
    get_prompt,
    get_diagnose_prompt,
    get_audit_prompt
)


@click.group()
@click.version_option()
@click.option("--short", is_flag=True, default=False, help="Disable verbose information of copilot execution steps")
@click.pass_context
def cli(ctx, short):
    init_openai()
    ctx.ensure_object(dict)
    ctx.obj["chain"] = CopilotLLM(verbose = not short)


@cli.command(help="execute operations based on prompt instructions")
@click.argument('instructions')
@click.pass_context
def execute(ctx, instructions):
    if click.confirm("Copilot may generate and execute inappropriate operations steps, are you sure to continue?"):
        chain = ctx.obj["chain"]
        result = chain.run(get_prompt(instructions))
        print(result)

@cli.command(help="diagnose problems for a Pod")
@click.argument('pod')
@click.argument('namespace', default="default")
@click.pass_context
def diagnose(ctx, namespace, pod):
    chain = ctx.obj["chain"]
    result = chain.run(get_diagnose_prompt(namespace, pod))
    print(result)

@cli.command(help="audit security issues for a Pod")
@click.argument('pod')
@click.argument('namespace', default="default")
@click.pass_context
def audit(ctx, namespace, pod):
    chain = ctx.obj["chain"]
    result = chain.run(get_audit_prompt(namespace, pod))
    print(result)


def main():
    cli()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import os
import openai


def init_openai():
    """
    Initializes the OpenAI API with the provided API key and sets the API base and version if using Azure.
    """
    if os.getenv("OPENAI_API_KEY") == "":
        raise Exception("Please set OPENAI_API_KEY via environment variable")

    if os.getenv("OPENAI_API_TYPE") == "azure":
        openai.api_type = "azure"
        openai.api_base = os.getenv("OPENAI_API_BASE")
        openai.api_version = "2023-03-15-preview"
        openai.api_key = os.getenv("OPENAI_API_KEY")
    else:
        openai.api_key = os.getenv("OPENAI_API_KEY")
        openai.api_base = "https://api.openai.com/v1"
